@@ -30,10 +30,10 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#define ID_SERIAL   0x8010
-#define ID_FIRMWARE 0x8011
-#define ID_RDBI     0x8021
-#define ID_HARDWARE 0x8050
+#define ID_SERIAL    0x8010
+#define ID_FIRMWARE  0x8011
+#define ID_LOGUPLOAD 0x8021
+#define ID_HARDWARE  0x8050
 
 #define PREDATOR 2
 #define PETREL   3
@@ -54,9 +54,6 @@ typedef struct shearwater_common_device_t {
 dc_status_t
 shearwater_common_setup (shearwater_common_device_t *device, dc_context_t *context, dc_iostream_t *iostream);
 
-dc_status_t
-shearwater_common_command (shearwater_common_device_t *device, const unsigned char input[], unsigned int isize);
-
 dc_status_t
 shearwater_common_transfer (shearwater_common_device_t *device, const unsigned char input[], unsigned int isize, unsigned char output[], unsigned int osize, unsigned int *actual);
 
