@@ -45,9 +45,9 @@
 #ifdef _WIN32
 #define NOGDI
 #endif
-#include <libusb-1.0/libusb.h>
+#include <libusb.h>
 #elif defined(USE_HIDAPI)
-#include <hidapi/hidapi.h>
+#include <hidapi.h>
 #endif
 
 #include <libdivecomputer/usbhid.h>
@@ -94,8 +94,10 @@ static dc_status_t dc_usbhid_iterator_next (dc_iterator_t *iterator, void *item)
 static dc_status_t dc_usbhid_iterator_free (dc_iterator_t *iterator);
 
 static dc_status_t dc_usbhid_set_timeout (dc_iostream_t *iostream, int timeout);
+static dc_status_t dc_usbhid_poll (dc_iostream_t *iostream, int timeout);
 static dc_status_t dc_usbhid_read (dc_iostream_t *iostream, void *data, size_t size, size_t *actual);
 static dc_status_t dc_usbhid_write (dc_iostream_t *iostream, const void *data, size_t size, size_t *actual);
+static dc_status_t dc_usbhid_ioctl (dc_iostream_t *iostream, unsigned int request, void *data, size_t size);
 static dc_status_t dc_usbhid_close (dc_iostream_t *iostream);
 
 typedef struct dc_usbhid_iterator_t {
@@ -137,15 +139,16 @@ static const dc_iterator_vtable_t dc_usbhid_iterator_vtable = {
 static const dc_iostream_vtable_t dc_usbhid_vtable = {
 	sizeof(dc_usbhid_t),
 	dc_usbhid_set_timeout, /* set_timeout */
-	NULL, /* set_latency */
 	NULL, /* set_break */
 	NULL, /* set_dtr */
 	NULL, /* set_rts */
 	NULL, /* get_lines */
 	NULL, /* get_available */
 	NULL, /* configure */
+	dc_usbhid_poll, /* poll */
 	dc_usbhid_read, /* read */
 	dc_usbhid_write, /* write */
+	dc_usbhid_ioctl, /* ioctl */
 	NULL, /* flush */
 	NULL, /* purge */
 	NULL, /* sleep */
@@ -685,6 +688,12 @@ dc_usbhid_set_timeout (dc_iostream_t *abstract, int timeout)
 	return DC_STATUS_SUCCESS;
 }
 
+static dc_status_t
+dc_usbhid_poll (dc_iostream_t *abstract, int timeout)
+{
+	return DC_STATUS_UNSUPPORTED;
+}
+
 static dc_status_t
 dc_usbhid_read (dc_iostream_t *abstract, void *data, size_t size, size_t *actual)
 {
@@ -773,4 +782,10 @@ out:
 
 	return status;
 }
+
+static dc_status_t
+dc_usbhid_ioctl (dc_iostream_t *abstract, unsigned int request, void *data, size_t size)
+{
+	return DC_STATUS_UNSUPPORTED;
+}
 #endif
